﻿/*
11/4/2008
  Adjacency List is looking good! 
  Please let me know if you find this useful at all:
     brian.stinar@cs.unm.edu
	 
	 Thanks and enjoy!
*/


package com.identityMine.cell
{
	import flash.display.MovieClip; // Everything inherits from this
	import flash.events.MouseEvent; // For reacting to mouse events
	import flash.events.KeyboardEvent; // For reacting to keyboard events
	import flash.ui.Keyboard; // For looking at individual keystrokes
	import flash.display.Stage; // For adding our keyboard listener to the stage
	
	// Arrows for visual debugging on graph connected-ness
	import Right_Arrow;
	import Left_Arrow;
	import Up_Arrow;
	import Down_Arrow;
	
	
	// This is the class representing a single cell in our maze
	public class CELL extends MovieClip
	{
		// We start off as not blocked
		private var blocked : Boolean = false;
		
		// Are we in debug mode?
		private var debug : Boolean = false;
		
		// Are we showing the arrows?
		private var showArrows : Boolean = false;
		
		// Are we in start, or end, selection mode?
		private var startSelection : Boolean = false;
		private var endSelection : Boolean = false;
		
		// Are we the first cell, the last cell?
		private var startCell : Boolean = false;
		private var endCell : Boolean = false;
		
		// If we have a right edge, this is the visual display of it
		private var RiArrow : Right_Arrow;
		//public var PossibleRightEdge : Boolean = false;
		
		// Left Edge
		private var LeArrow : Left_Arrow;
		//private var PossibleLeftEdge : Boolean = false;
		
		// Up Edge
		private var UpArrow : Up_Arrow;
		//private var PossibleUpEdge : Boolean = false;
		
		// Down Edge
		private var DownArrow : Down_Arrow;
		//private var PossibleDownEdge : Boolean = false;
		
		private var rows : int;
		private var columns : int;
		
		// I didn't realize that these EXPLICITLY needed a reference to stage from above...
		private var MyStage:Stage;
		
		private var iterator;
		
		/* 
			These are data elements required to actually run A* 
		*/
		
		// Adjacency List
		public var EdgeList : Array;
		
		// Total Node List
		private var TotalNodeList: Array;
		
		// Used to keep track of our position in terms of grid location
		public var iPos : int;
		public var jPos : int;
		
		public var parentCell : CELL; // This is used in backtracking after I have completed a search
		
		// Quick method for getting VALUE of startCell, since if I actually returned startCell, someone (me) could mess it up and change
		// the value on accident, since everything is by reference in ActionScript
		public function getStartCell() : Boolean{
			if (startCell == true){
				return true;
			}
			if (startCell == false){
				return false;
			}
			throw new Error("startCell neither true nor false, something is messed up");
		} // end getStartCell
		
		
		// Quick method for getting VALUE of endCell 
		public function getEndCell() : Boolean{
			if (endCell == true){
				return true;
			}
			if (endCell == false){
				return false;
			}
			throw new Error("endCell neither true nor false, something is messed up");
		} // end getEndCell
		
		
		// Method for debug tracing
		private function dtrace(input : String){
			if (debug == true){
				trace(input);
			}
		}

		
		// Our constructor
		public function CELL(){
			blocked = false;
			addEventListener(MouseEvent.MOUSE_DOWN, LeftClickHandler);
			EdgeList = new Array();
		} // end function Cell()

		
		// Since we cannot add additional arguments to the constructor, I need to do this directly after 
		// calling the constructor
		public function SetStage(inputStage : Stage){
			
			// Keep a reference to the stage
			MyStage = inputStage;
			// Setup our listeners to the stage
			MyStage.addEventListener(KeyboardEvent.KEY_DOWN, KeyboardHandler);
		} // end function SetStage()
		
		
		// We need a reference to all the nodes, so I can remove myself from everyone's adjacency list
		// whenever I become blocked
		
		public function setNodeList(input : Array){
			TotalNodeList = input;
		}
		
		public function setRowsColumns(inputRows: int, inputColumns : int){
			rows = inputRows;
			columns = inputColumns;
		}
		
		public function partOfPath():void{
			gotoAndPlay(150);
		}
		
		public function RemoveFromAdjacency(inputI : int, inputJ: int):Boolean{
			
			// iterate through my list of adjacent edges.
			// if I find the one I am looking for, remove it and return true.
			// otherwise return false;
			
			var found : Boolean = false;
			
			if (EdgeList.length > 4){
				throw new Error("ERROR = EdgeList.length > 4");
			}
					
			for (iterator = 0; iterator < EdgeList.length; iterator ++){
				if ((EdgeList[iterator].iPos == inputI) && (EdgeList[iterator].jPos == inputJ)){
					//array.splice(array.indexOf(obj),1); <- Remove an object from an array, thanks to:
					// http://www.razorberry.com/blog/archives/2006/10/27/learning-as3-1/
					
					EdgeList.splice(iterator, 1);
					
					// We have already found a copy of this in the adjacency list - we have a duplicate and 
					// a problem with how I am constructing the adjacency list
					
					if (found == true){
						throw new Error("Duplicate entry found in Adjacency List. This is a serious problem");
					}
					
					found = true;
				} // end if
			} // end for 
			
			return found; 
		} // end RemoveFromAdjacency()
	
		public function SetIJ(inputI : int, inputJ : int){
			iPos = inputI;
			jPos = inputJ;
		}
	
	
	
		function KeyboardHandler(event:KeyboardEvent):void{
		    dtrace("Keyboard seen");
			
			if ((event.keyCode == Keyboard.SPACE) || (event.keyCode == Keyboard.ENTER))
			{
				
				if ((startSelection == false) && (endSelection == false)){
					startSelection = true;
					endSelection = false;
					dtrace("startSelection = " + startSelection + " endSelection = " + endSelection);
					return
				}
				
				if ((startSelection == true) && (endSelection == false)){
					startSelection = false;
					endSelection = true;
					dtrace("startSelection = " + startSelection + " endSelection = " + endSelection);
					return;
				}
							
				if ((startSelection == false) && (endSelection == true)){
					startSelection = false;
					endSelection = false;
					dtrace("startSelection = " + startSelection + " endSelection = " + endSelection);
					return;
				}
				
				if ((startSelection == true) && (endSelection == true)){
					
					// We should never reach here
					 throw new Error("We should never be in startSelection == true AND endSelection == true!");
				}
				// We should also never reach here
				throw new Error("Unanticipated State");		
			}

		} // end KeyboardHandler()
	
	
		// This method is called whenever we have a mouse click on our square
		function LeftClickHandler(event:MouseEvent):void{
			dtrace("mouse click received");

			// if we are not blocked, then we become blocked on a single left click
			// only IF we are not a start or end cell
			
			if ((blocked == false) && (startSelection == false) && (endSelection == false) && (startCell == false) && (endCell == false)){
				
				
				
				// if I become blocked, I need to remove myself from the adjacency list of my neighbors, and set their
				// corresponding PossibleDirectionEdge = false
				
				// The simple way to do this is to take advantage of the fact that I do NOT have a general graph,
				// the most neighbors any cell can have is for. I can just inspect my I and J, and determine which neighbors
				// I will have. If I have them as a neighbor, I need to remove their corresponding arrow INTO me.
				
				// I need to be removing elements based on their absense or presence in my adjacency list
				
				//var Neighbors: Array = getNeighbors();
				
				
				var i = 0;
				
				
				// I WAS CHANGING THE EdgeList LENGTH WHILE ITERATING OVER IT! 
				var oldLength : int = EdgeList.length;
				var oldEdgeList : Array = new Array();
				
				for (i = 0; i < oldLength; i++){
					oldEdgeList.push(EdgeList[i]);
				}
				
				for (i = 0; i < oldEdgeList.length; i++){
					//dtrace("		TotalNodeList[...].edgeList["+i+"].iPos="+TotalNodeList[iPos * columns + jPos].EdgeList[i].iPos);
					//dtrace("		TotalNodeList[...].edgeList["+i+"].jPos="+TotalNodeList[iPos * columns + jPos].EdgeList[i].jPos);
					RemoveArrow(oldEdgeList[i]);
				}
				
				
				
				// I need to look at the EdgeList of all my neighbors.
				// if I am in their EdgeList, I need to remove myself from it.
				// I also need to remove myself from their arrows
				
				var Neighbors : Array = getNeighbors();
				// Go through all my neighbors
				// if the neighbor is not blocked, remove me from their adjacency list
				
				
				
				for (i = 0; i < Neighbors.length; i++){
					// If the neighbor is not blocked
					if (Neighbors[i].blocked == false){
						Neighbors[i].RemoveArrow(this);
							
					} // end neighbor not blocked
				} // end for loop
				
				
				gotoAndPlay(2);	
				blocked = true;
				return;
			}
			
			
			// if we were already blocked, then we become unblocked
			// only IF we are not a start of end cell
			if ((blocked == true) && (startSelection == false) && (endSelection == false) && (startCell == false) && (endCell == false) ){
	
				var myNeighbors : Array = getNeighbors();
				dtrace ("Adding Arrows");
				dtrace ("iPos = " + iPos);
				dtrace ("jPos = " + jPos);
				
				dtrace ("myNeighbors.length=" + myNeighbors.length);
				for (i = 0; i < myNeighbors.length; i++){
					if (myNeighbors[i].blocked == false){
						dtrace("	unblocked	Neighbors[" + i + "].iPos= " + myNeighbors[i].iPos);
						dtrace("	unblocked	Neighbors[" + i + "].jPos= " + myNeighbors[i].jPos);
						AddArrow(myNeighbors[i]);
					}
					if (myNeighbors[i].blocked == true){
						dtrace("	blocked	Neighbors[" + i + "].iPos= " + myNeighbors[i].iPos);
						dtrace("	blocked	Neighbors[" + i + "].jPos= " + myNeighbors[i].jPos);
					}
				}
				
				// When we become unblocked, we also need
				// to check to see if our neighboring nodes can now 
				// have an edge going into us
				
				
				// if left neighbor exists,
				if (iPos > 0){
					// if left neighbor is unblocked, we need to make it so it has an arrow to us
					if (TotalNodeList[((iPos-1) * columns) + jPos].blocked == false){
						TotalNodeList[((iPos-1) * columns + jPos)].AddArrow(this);
					}
				}
				
				// we have a right edge
				if (iPos < columns - 1){
					if (TotalNodeList[((iPos+1) * columns) + jPos].blocked == false){
					TotalNodeList[((iPos+1) * columns) + jPos].AddArrow(this);
					}
				}
						
				// we have an up edge
				if (jPos > 0){
					if (TotalNodeList[iPos * columns + jPos - 1 ].blocked == false){
						TotalNodeList[iPos * columns + jPos - 1 ].AddArrow(this);
					}
				}
						
				// we have a down edge
				if (jPos < rows - 1){
					if (TotalNodeList[iPos * columns + jPos + 1].blocked == false){
						TotalNodeList[iPos * columns + jPos + 1 ].AddArrow(this);
					}
				}
				
				
				gotoAndPlay(26);
				blocked = false;
				return;
				
			} // end check for unblock
			
			// If we are in selecting start position mode...
			
			// And we are NOT currently selected as a start cell, make us one
			if ((blocked == false) && (startSelection == true) && (startCell == false) && (endCell == false)){
				startCell = true;
				gotoAndPlay(50);
				return;
			}
			
			// If we ARE currently selected as a start cell, and we are in start cell selection 
			// mode, unselect us
			if ((blocked == false) && (startSelection == true) && (startCell == true) && (endCell == false)){
				startCell = false;
				gotoAndPlay(75);
				return;
			}
			
			// If we select you to be an end cell
			if ((blocked == false) && (startSelection == false) && (endSelection == true) && (endCell == false) && (startCell == false))
			{
				endCell = true;
				gotoAndPlay(100);
				return;
			}
			
			if ((blocked == false) && (startSelection == false) && (endSelection == true) && (endCell == true))
			{
				endCell = false;
				gotoAndPlay(125);
				return;
			}
			
			
		} // end function LeftClickHandler()
		
		private function getNeighbors():Array{
				
			var Neighbors : Array = new Array();
						
			// we have a left edge
			if (iPos > 0){
				Neighbors.push(TotalNodeList[((iPos-1) * columns) + jPos]);
			}
						
			// we have a right edge
			if (iPos < columns - 1){
				Neighbors.push(TotalNodeList[((iPos + 1) * columns) + jPos]);
			}
						
			// we have an up edge
			if (jPos > 0){
				Neighbors.push(TotalNodeList[iPos * columns + jPos - 1 ]);
			}
						
			// we have a down edge
			if (jPos < rows - 1){
				Neighbors.push(TotalNodeList[iPos * columns + jPos + 1]);
			}
					
			return Neighbors;
					
		} // end getNeighbors()
		
		private function RemoveArrow(inputCell : CELL){
			dtrace("inside RemoveArrow");
			if ((iPos == inputCell.iPos) && (jPos == inputCell.jPos)){
				//throw new Error("Attempting to remove a non-existing edge from yourself to yourself");
				return;
			}
		
			dtrace("From iPos = " + iPos);
			dtrace("From iPos = " + jPos);
			dtrace("	removing Cell inputCell.iPos = " + inputCell.iPos);
			dtrace("	removing Cell inputCell.jPos = " + inputCell.jPos);
			
			RemoveFromAdjacency(inputCell.iPos, inputCell.jPos);
		
			// inputCell is below us
			if ((iPos == inputCell.iPos) && (jPos == inputCell.jPos - 1)){
				dtrace("removing down arrow");
				if (showArrows == true){
					removeChild(DownArrow);
				}
				return;
			}
			
			// inputCell is above us
			if ((iPos == inputCell.iPos) && (jPos == inputCell.jPos + 1)){
				dtrace("removing up arrow");
				if (showArrows == true){
					removeChild(UpArrow);
				}
				return;
			}
			
			// input cell is to the right
			if ((iPos == inputCell.iPos -1) && (jPos == inputCell.jPos)){
				dtrace("removing right arrow");
				if (showArrows == true){
					removeChild(RiArrow);
				}
				return;
			}
		
			// input cell is to the left
			if ((iPos == inputCell.iPos + 1) && (jPos == inputCell.jPos)){
				dtrace("removing left arrow");
				if (showArrows == true){
					removeChild(LeArrow);
				}
				return;
			}
		
			throw new Error("Cannot Remove an Arrow from a non-adjacent cell.");
		} // end RemoveArrow
		
		public function AddArrow(inputCell : CELL){
			
			if ((iPos == inputCell.iPos) && (jPos == inputCell.jPos)){
				throw new Error("Attempting to add a non-existing edge from yourself");
				return;
			}
			
			//inputCell is above us
			if ((iPos == inputCell.iPos) && (jPos == inputCell.jPos + 1)){
				if (UpArrow == null){
					UpArrow = new Up_Arrow();
					UpArrow.y = UpArrow.y - 300;
				}
				AddToEdgeListSafe(TotalNodeList[((inputCell.iPos) * columns) + inputCell.jPos]);
 				if (showArrows == true){
					addChild(UpArrow);
				}
				return;
			}
			
			// inputCell is below us
			if ((iPos == inputCell.iPos) && (jPos == inputCell.jPos - 1)){
				if (DownArrow == null){
					DownArrow = new Down_Arrow();		
					DownArrow.y = DownArrow.y + 600;
					DownArrow.x = DownArrow.x + 200;
				}
				AddToEdgeListSafe(TotalNodeList[((inputCell.iPos) * columns) + inputCell.jPos]);
				if (showArrows == true){
					addChild(DownArrow);
				}
				return;
			}
			
			// input cell is to the left
			if ((iPos == inputCell.iPos +1) && (jPos == inputCell.jPos)){
				if (LeArrow == null){
					LeArrow = new Left_Arrow();
					LeArrow.x = LeArrow.x - 400;
					LeArrow.y = LeArrow.y + 450;
				}
				AddToEdgeListSafe(TotalNodeList[((inputCell.iPos) * columns) + inputCell.jPos]);
				if (showArrows == true){
					addChild(LeArrow);
				}
				return;
			}
		
			// input cell is to the right
			if ((iPos == inputCell.iPos -1) && (jPos == inputCell.jPos)){
				if (RiArrow == null){
					RiArrow = new Right_Arrow();
					RiArrow.x = RiArrow.x + 600;
				}
				
				// Add the right edge to my adjacency list
				AddToEdgeListSafe(TotalNodeList[((inputCell.iPos) * columns) + inputCell.jPos]);
				if (showArrows == true){
					addChild(RiArrow);
				}
				return;
			}
			throw new Error("Unforseen Circumstance");
		}
		
		// Method to add a cell to our edge list, only if it is already not in the list
		private function AddToEdgeListSafe(inputCell : CELL){
			
			for (var iterator = 0; iterator < EdgeList.length; iterator++){
				if (EdgeList[iterator] == inputCell){
					//trace("With cell.iPos : " + iPos);
					//trace("With cell.jPos : " + jPos);
					
					//trace("		Error adding inputCell.ipos : " + inputCell.iPos);
					//trace("		Error adding inputCell.jpos : " + inputCell.jPos);
					//throw new Error("Already Added to Neighbor");
					return;
				}
			}
			EdgeList.push(inputCell);
		}
		
		public function test(){
			gotoAndPlay(50);
		}
		
		public function test2(){
			gotoAndPlay(100);
		}
		
	} // end class CELL

} // end package com.identityMine.cell
